<?php

namespace SupNext\Controllers;

use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Service\CRUD as CrudService;
use SupNext\Util\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class LocalVarController {
  private $environment;
  private $crudService;
  private $util;

  public function __construct(Environment $environment, CrudService $crudService, Util $util) {
    $this->environment  = $environment;
    $this->crudService  = $crudService;
    $this->util         = $util;
  }

  public function getLocalVar(Request $request, Response $response) {
    try {
      $data["CDFILCENTCMP"]      = $this->util->getcentralCompras();
      $data["CDOPERADOR"]        = $this->environment->getCdoperador();
      $data["CDFILIAL"]          = $this->environment->getCdFilial();
      $data["NRORG"]             = $this->environment->getNrorg();
      $data["IDSISDOS"]          = $this->getIdSisDos($data);
      $data["REGISTERED_PARAMS"] = $this->getRegisteredParams($data);

      $response->addDataSet(new DataSet("data", $data));
    } catch (\Exception $exception) {
      throw new \Exception($exception->getMessage());
    }
  }

  private function getRegisteredParams(array $data): array {
    $conditions = $this->util->createConditions([
      "CDFILCENTCMP" => $data["CDFILCENTCMP"],
      "NRORG"        => $data["NRORG"]
    ]);

    $registeredParams = $this->crudService
      ->getFilteredDataSet("registeredParams", $conditions)
      ->getRows()[0];

    return $registeredParams;
  }

  private function getIdSisDos($data) {
    try {
      $conditions = [
        "NRORG" => $data["NRORG"]
      ];

      $id = $this->util->filteredDataSet($conditions, "getIdSisDos")[0]["IDSISDOS"];

      return $id;
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao pegar idSisDos: " . $exception->getMessage(), 500);
    }
  }
}
