<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use Teknisa\Libs\Util\Environment;
use SupNext\Util\Util;
use SupNext\Services\QuoteSheetGrid\AddSupplierService as Service;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class AddSupplierController {
  private $environment;
  private $util;
  private $service;

  public function __construct(Environment $environment, Util $util, Service $service) {
    $this->environment = $environment;
    $this->util        = $util;
    $this->service     = $service;
  }

 public function addSupplier(Request $request, Response $response) {
    try{
      $data = $request->getParameters();
      $data["NRORG"] = $this->environment->getNrorg();

      $supplierExist = $this->verifySupplierExist($data);

      if(!empty($supplierExist)) return $response->setError(new Error("sup::SUPPLIER_ALREADY_REGISTERED", 400));

      $this->validateSupplier($data, $response);
      $procResult = $this->service->procIncFornCota($data);

      if ($procResult['P_IDERRO'] == "000") {
        return $response->addDataSet(new DataSet("addSupplier", ["message" => "Inclusão de fornecedor realizada com sucesso.", "status_code" => 200]));
      }


      $logErrors = $this->getLogErrors($data);

      $response->setError(new Error($logErrors, 400));
      $this->service->deleteLogErrors($data);
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao adicionar fornecedor: " . $exception->getMessage(), 500);
    }
  }

  private function verifySupplierExist(array $data): array {
    $conditionsArray = [
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFORNECED"  => $data["CDFORNECED"],
      "NRORG"       => $data["NRORG"]
    ];

    $supplierExist = $this->util->filteredDataSet($conditionsArray, "verifySupplierExist");

    return $supplierExist;
  }

  private function validateSupplier(array $data, Response $response) {
    $conditionsArray = [
      "CDFORNECED"   => $data["CDFORNECED"],
      "NRINSJURFORN" => $data["NRINSJURFORN"],
      "NRINSJURFORN" => $data["NRINSJURFORN"],
      "NRORG"        => $data["NRORG"]
    ];

    $supplierDetails = $this->util->filteredDataSet($conditionsArray, "validateSupplierQuote")[0];

    if (!in_array($data["CDFORNECED"], $supplierDetails)) return $response->setError(new Error("Fornecedor " . $data["NMFANTFORN"] . " não cadastrado para o produto da cotação.", 400));
  }

  private function getLogErrors(array $data): array {
    $conditionsArray = [
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "NRORG"       => $data["NRORG"]
    ];

    $logErrors = $this->util->filteredDataSet($conditionsArray, "logErrors");

    return $logErrors;
  }
}