<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use SupNext\Services\QuoteSheetGrid\ChangeValidityService as Service;
use SupNext\Util\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class ChangeValidityController {
    private $service;
    private $util;

    public function __construct(Service $service, Util $util) {
        $this->service = $service;
        $this->util    = $util;
    }

    public function changeValidity(Request $request, Response $response) {
        $data = $request->getParameters()["params"];
        
        try {
            $quotesValidity = $this->getQuotesValidity($data);

            foreach ($quotesValidity as $quote) {
                $data["CDFILIAL"]    = $quote["CDFILIAL"];
                $data["NRCOLEPRECO"] = $quote["NRCOLEPRECO"];

                $this->service->updateValidityCollection($data);
                $this->service->updateValidityItemCole($data);
            }

            return $response->addDataSet(new DataSet("changeValidity", ["message" => "Validade alterada com sucesso.", "status_code" => 200]));
        } catch (\Exception $exception) {
            throw new \Exception("Erro ao alterar validade: " . $exception->getMessage(), 500);
        }
    }

    private function getQuotesValidity(array $data): array {
        try {
            $conditions = [
                "DTINIANTERIOR"   => $data["DTINIVALCOL"],
                "DTFINANTERIOR"   => $data["DTFINVALCOL"],
                "CDCOMPRADOR"     => $data["CDCOMPRADOR"],
                "CDCOMPRADOR"     => $data["CDCOMPRADOR"],
                "COTACAO"         => $data["NRCOLEPRECO"],
                "COTACAO"         => $data["NRCOLEPRECO"],
                "CDFILCENTCMP"    => $data["CDFILCENTCMP"],
                "NRORG"           => $data["NRORG"],
                "CDFILCENTCMP"    => $data["CDFILCENTCMP"],
                "CDFORNECED"      => $data["CDFORNECED"],
                "CDFORNECED"      => $data["CDFORNECED"],
                "DTINIANTERIOR"   => $data["DTINIVALCOL"],
                "DTFINANTERIOR"   => $data["DTFINVALCOL"],
                "NRORG"           => $data["NRORG"]
            ];
            $quotesValidity = $this->util->filteredDataSet($conditions, "getQuotesValidity");

            return $quotesValidity;
        } catch (\Exception $exception) {
            throw new \Exception("Erro ao pegar cotações com validades: " . $exception->getMessage(), 500);
        }
    }
}
