<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use SupNext\Services\QuoteSheetGrid\SaveAdditionalDataService as Service;
use SupNext\Controllers\QuoteSheetGrid\AddRegionController as AddRegion;
use SupNext\Util\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class SaveAdditionalDataController {
  private $service;
  private $addRegion;
  private $util;

  public function __construct(Service $service, AddRegion $addRegion, Util $util) {
    $this->service   = $service;
    $this->addRegion = $addRegion;
    $this->util      = $util;
  }

  public function saveAdditionalData(Request $request, Response $response) {
    $data = $request->getParameters()["params"];

    try {
      $this->validateSpreadsheet($data);
      $this->addRegion->executeProcedureItemcol($data);

      $this->service->deletePaymentDeadline($data);
      $this->service->updateItemCole($data);
      $this->service->updateFreightSupplier($data);

      $query = $this->service->generateUpdateQuery($data);

      $this->service->updateItemColeCustom($query, $data);

      return $response->addDataSet(new DataSet("saveAdditionalData", ["message" => "Atualização dos dados adicionais realizada com sucesso.", "status_code" => 200]));
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao salvar informações dados adicionais: " . $exception->getMessage(), 500);
    }
  }

  private function validateSpreadsheet(array $data): void {
    $this->verifyCollectionApr($data);
    $this->searchBrand($data);
    $this->getChargeType($data);
    $this->getSumPercentage($data);
  }

  private function verifyCollectionApr(array $data): void {
    $conditions = [
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "NRORG"       => $data["NRORG"]
    ];

    $collectionApr = $this->util->filteredDataSet($conditions, "verifyCollectionApr");

    if (empty($collectionApr)) {
      throw new \Exception("sup::ERROR_INFORMATION_WORKSHEET");
    }
  }

  private function searchBrand(array $data): void {
    $conditions = [
      "CDARVPROD" => $data["CDARVPROD"],
      "CDMARCA"   => $data["CDMARCA"],
      "CDMARCA"   => $data["CDMARCA"],
      "NRORG"     => $data["NRORG"]
    ];

    $brand = $this->util->filteredDataSet($conditions, "distinctBranchProduct");

    if (empty($brand)) {
      throw new \Exception("sup::BRAND_NOT_REGISTERED_OR_QUALIFIED_FOR_PRODUCT");
    }
  }

  private function getChargeType(array $data): void {
    if ($data["CDTIPCOB"]) {
      $conditions = [
        "CDTIPCOB" => $data["CDTIPCOB"],
        "CDTIPCOB" => $data["TCDTIPCOB"],
        "NRORG"    => $data["NRORG"]
      ];

      $chargeType = $this->util->filteredDataSet($conditions, "chargeType");

      if (empty($chargeType)) {
        throw new \Exception("sup::BILLING_NOT_REGISTERED");
      }
    }
  }

  private function getSumPercentage(array $data): void {
    $conditions = [
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFORNECED"  => $data["CDFORNECED"],
      "CDPRODUTO"   => $data["CDPRODUTO"],
      "CDMARCA"     => $data["CDMARCA"],
      "CDREGIAO"    => $data["CDREGIAO"],
      "NRORG"       => $data["NRORG"]
    ];

    $sumPercentage = $this->util->filteredDataSet($conditions, "getSumPercentage");

    if (!empty($sumPercentage) && $sumPercentage[0]["SOMAPERC"] != 100 && $sumPercentage[0]["SOMAPERC"] != null) {
      throw new \Exception("sup::PERCENTAGE_PAYMENT_100");
    }
  }
}
