<?php

namespace SupNext\Services\QuoteSheetGrid;

use Doctrine\DBAL\Connection;
use SupNext\Util\Util;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class AddQuotationService {
  private $connection;
  private $util;

  public function __construct(Connection $connection, Util $util) {
    $this->connection = $connection;
    $this->util       = $util;
  }

  public function saveQuotation(array $data) {
    try {
      $query = "INSERT INTO COLETAPR
                  (
                      CDFILIAL,
                      NRCOLEPRECO,
                      NRSEMAINIC,
                      NRSEMAFINAL,
                      DTINIVALCOL,
                      DTFINVALCOL,
                      CDOPERADOR,
                      DTCOLEPREC,
                      IDDISMONTLIS,
                      CDOPERINCL,
                      DTINCLCOLET,
                      DSOBSCOTPREC,
                      IDSITUCOLETA,
                      NRCOMPRA,
                      NRORG,
                      CDTPINDICECOL,
                      IDPERVALORCOL,
                      CDINDICECOL,
                      DTREFINDCOL
                  )
                  VALUES
                  (
                      :P_CDFILIAL,
                      :P_NRCOLEPRECO,
                      :P_NRSEMAINIC ,
                      :P_NRSEMAFINAL,
                      :P_DTINIVALCOL,
                      :P_DTFINVALCOL,
                      :P_CDOPERADOR,
                      :P_DTCOLEPREC,
                      :P_IDDISMONTLIS,
                      :P_CDOPERINCL,
                      SYSDATE,
                      :P_DSOBSCOTPREC,
                      :P_IDSITUCOLETA,
                      :P_NRCOMPRA,
                      :P_NRORG,
                      :P_CDTPINDICE,
                      :P_IDPERVALOR,
                      :P_CDINDICE,
                      :P_DTREFIND
      )";

      $params = [
        "P_CDFILIAL"     => $data["CDFILIAL"],
        "P_NRCOLEPRECO"  => $data["NRCOLEPREC"],
        "P_NRSEMAINIC"   => $data["NRSEMAINIC"],
        "P_NRSEMAFINAL"  => $data["NRSEMAFINAL"],
        "P_DTINIVALCOL"  => $data["DTINIVALCOL"],
        "P_DTFINVALCOL"  => $data["DTFINVALCOL"],
        "P_CDOPERADOR"   => $data["CDOPERADOR"],
        "P_DTCOLEPREC"   => $data["DTCOLEPREC"],
        "P_IDDISMONTLIS" => $data["IDDISMONTLIS"],
        "P_CDOPERINCL"   => $data["CDOPERINCL"],
        "P_DSOBSCOTPREC" => $data["DSOBSCOTPREC"],
        "P_IDSITUCOLETA" => $data["IDSITUCOLETA"],
        "P_NRCOMPRA"     => $data["NRCOMPRA"],
        "P_NRORG"        => $data["NRORG"],
        "P_CDTPINDICE"   => $data["CDTPINDICE"],
        "P_IDPERVALOR"   => $data["IDPERVALOR"],
        "P_CDINDICE"     => $data["CDINDICE"],
        "P_DTREFIND"     => $data["DTREFIND"]
      ];

      return $this->util->crud($query, $params);
    } catch (\Exception $exception) {
      throw new \Exception($exception->getMessage(), 500);
    }
  }

  public function procedureIncludesItemcolTodos(array $data) {
    try {
      $procedure = new StoredProcedure($this->connection, "INCLUI_ITEMCOL_TODOS");

      $procedure->addParam(new Param("P_CDFILIAL",    Param::PARAM_INPUT, $data["CDFILIAL"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFILIPROC",  Param::PARAM_INPUT, $data["CDFILIAL"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRCOLEPRECO", Param::PARAM_INPUT, $data["NRCOLEPREC"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_DTINI",       Param::PARAM_INPUT, $data["DTINIVALCOL"], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_DTFIN",       Param::PARAM_INPUT, $data["DTFINVALCOL"], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_PRODINI",     Param::PARAM_INPUT, $data["CDPRODINI"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_PRODFIN",     Param::PARAM_INPUT, $data["CDPRODFIN"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDREGIAO",    Param::PARAM_INPUT, $data["CDREGIAO"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFORNECED",  Param::PARAM_INPUT, $data["CDFORNECED"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDMARCA",     Param::PARAM_INPUT, $data["CDMARCA"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CONSIST",     Param::PARAM_INPUT, $data["CDVERIF"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_IDULTPREC",   Param::PARAM_INPUT, $data["IDCARGULTC"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDOPERINCL",  Param::PARAM_INPUT, $data["CDOPERINCL"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_ROWSAFFECT",  Param::PARAM_OUTPUT, NULL,                Param::PARAM_TYPE_STR));

      return $procedure->execute();
    } catch (\Exception $exception) {
      throw new \Exception($exception->getMessage(), 500);
    }
  }

  public function procedureIncludesItemcolAComprar(array $data) {
    try {
      $procedure = new StoredProcedure($this->connection, "INCLUI_ITEMCOL_ACOMPRAR");

      $procedure->addParam(new Param("P_CDFILIAL",    Param::PARAM_INPUT, $data["CDFILIAL"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFILIPROC",  Param::PARAM_INPUT, $data["CDFILIAL"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRCOLEPRECO", Param::PARAM_INPUT, $data["NRCOLEPREC"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_DTINI",       Param::PARAM_INPUT, $data["DTINIVALCOL"], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_DTFIN",       Param::PARAM_INPUT, $data["DTFINVALCOL"], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_PRODINI",     Param::PARAM_INPUT, $data["CDPRODINI"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_PRODFIN",     Param::PARAM_INPUT, $data["CDPRODFIN"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDREGIAO",    Param::PARAM_INPUT, $data["CDREGIAO"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFORNECED",  Param::PARAM_INPUT, $data["CDFORNECED"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDMARCA",     Param::PARAM_INPUT, $data["CDMARCA"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CONSIST",     Param::PARAM_INPUT, $data["CDVERIF"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_IDULTPREC",   Param::PARAM_INPUT, $data["IDCARGULTC"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDOPERINCL",  Param::PARAM_INPUT, $data["CDOPERINCL"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_ROWSAFFECT",  Param::PARAM_OUTPUT, NULL,                Param::PARAM_TYPE_STR));

      return array($procedure->execute());
    } catch (\Exception $exception) {
      throw new \Exception($exception->getMessage(), 500);
    }
  }

  public function procedureIncludesItemcolProdCompra(array $data) {
    try {
      $procedure = new StoredProcedure($this->connection, "INCLUI_ITEMCOL_PRODCOMPRA");

      $procedure->addParam(new Param("P_CDFILIAL",    Param::PARAM_INPUT, $data["CDFILIAL"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFILIPROC",  Param::PARAM_INPUT, $data["CDFILIAL"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRCOLEPRECO", Param::PARAM_INPUT, $data["NRCOLEPREC"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_DTINI",       Param::PARAM_INPUT, $data["DTINIVALCOL"], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_DTFIN",       Param::PARAM_INPUT, $data["DTFINVALCOL"], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_PRODINI",     Param::PARAM_INPUT, $data["CDPRODINI"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_PRODFIN",     Param::PARAM_INPUT, $data["CDPRODFIN"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDREGIAO",    Param::PARAM_INPUT, $data["CDREGIAO"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFORNECED",  Param::PARAM_INPUT, $data["CDFORNECED"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDMARCA",     Param::PARAM_INPUT, $data["CDMARCA"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CONSIST",     Param::PARAM_INPUT, $data["CDVERIF"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_IDULTPREC",   Param::PARAM_INPUT, $data["IDCARGULTC"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDOPERINCL",  Param::PARAM_INPUT, $data["CDOPERINCL"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRCOMPRA",    Param::PARAM_INPUT, $data["NRCOMPRA"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_ROWSAFFECT",  Param::PARAM_OUTPUT, NULL,                Param::PARAM_TYPE_STR));

      return $procedure->execute();
    } catch (\Exception $exception) {
      throw new \Exception($exception->getMessage(), 500);
    }
  }

  public function procedureCargDescSolCot(array $data): void {
    try {
      $procedure = new StoredProcedure($this->connection, "CARG_DESCSOL_COT");

      $procedure->addParam(new Param("P_CDFILIAL",    Param::PARAM_INPUT, $data["CDFILIAL"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRCOLEPRECO", Param::PARAM_INPUT, $data["NRCOLEPREC"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_DTINI",       Param::PARAM_INPUT, $data["DTINIVALCOL"], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_DTFIN",       Param::PARAM_INPUT, $data["DTFINVALCOL"], Param::PARAM_TYPE_STR));

      $procedure->execute();
    } catch (\Exception $exception) {
      throw new \Exception($exception->getMessage(), 500);
    }
  }
}
