<?php

namespace SupNext\Services\QuoteSheetGrid;

use SupNext\Util\Util;

class PriceAdjustmentService {
    private $util;

    public function __construct(Util $util) {
        $this->util = $util;
    }

    public function insertItemCotacaoAdjustment($data) {
        try {
            $query = "INSERT INTO MOVOPITEMCOT
                                  (
                                        CDFILIALMCOT,
                                        NRCOLEPRECOMCOT,
                                        CDMARCAMCOT,
                                        CDREGIAOMCOT,
                                        CDFORNECEDMCOT,
                                        CDPRODUTOMCOT,
                                        DTOPERAITMCOT,
                                        CDOPERADORMCOT,
                                        DSALTERITMCOT,
                                        DSJUSTITMCOT,
                                        DSRDMUSUMCOT,
                                        DSENDUSUMCOT,
                                        DSNMUSUMCOT,
                                        DSMETUSUMCOT
                                    ) SELECT CDFILIAL, NRCOLEPRECO, CDMARCA, CDREGIAO,
                                             CDFORNECED, CDPRODUTO, SYSDATE, :P_CDOPERADOR,
                                             :P_DESCRICAO || TO_CHAR(ITEMCOLE.VRPRECUNIITM) ||' / '||TO_CHAR(ITEMCOLE.VRPRECUNIITM+(ITEMCOLE.VRPRECUNIITM*(:P_PORCENT / 100))),
                                             :P_DESCRICAO, :P_DSRDMUSUMCOT, :P_DSENDUSUMCOT,
                                             :P_DSNMUSUMCOT, :P_DSMETUSUMCOT
                                      FROM ITEMCOLE
                                      WHERE ITEMCOLE.CDFILIAL = :P_CDFILCENT
                                            AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO
                                            AND ITEMCOLE.NRORG = :P_NRORG
                                            AND EXISTS(SELECT PRODUTO.CDPRODUTO
                                                       FROM PRODUTO
                                                       WHERE ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO
                                                            AND PRODUTO.NRORG = :P_NRORG
                                                            AND (PRODUTO.CDARVPROD BETWEEN :P_CDINI AND :P_CDFIN OR :P_TODOS = 'T'))";

            $param = [
                "P_CDOPERADOR"   => $data["CDOPERINCL"],
                "P_DESCRICAO"    => $data["DESCRICAO"],
                "P_PORCENT"      => $data["REAJUSTE"],
                "P_DSRDMUSUMCOT" => $data["DSRDMUSUMCOT"],
                "P_DSENDUSUMCOT" => $data["DSENDUSUMCOT"],
                "P_DSNMUSUMCOT"  => $data["DSNMUSUMCOT"],
                "P_DSMETUSUMCOT" => $data["DSMETUSUMCOT"],
                "P_CDFILCENT"    => $data["CDFILCENT"],
                "P_NRCOLEPRECO"  => $data["NRCOLEPRECO"],
                "P_CDINI"        => $data["CDPRODINI"],
                "P_CDFIN"        => $data["CDPRODFIN"],
                "P_TODOS"        => $data["TODOSPROD"],
                "P_NRORG"        => $data["NRORG"]
            ];

            return $this->util->crud($query, $param);
        } catch (Exception $exception) {
            throw new \Exception("Erro ao inserir preço reajustado item cole: " . $exception->getMessage(), 500);
        }
    }

    public function updatePriceDate($data) {
        try {
            $query = "UPDATE ITEMCOLE
                      SET ITEMCOLE.VRPRECUNIITM = ITEMCOLE.VRPRECUNIITM+(ITEMCOLE.VRPRECUNIITM*(:P_PORCENT / 100))
                      WHERE ITEMCOLE.CDFILIAL = :P_CDFILCENT
                            AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO
                            AND ITEMCOLE.NRORG = :P_NRORG
                            AND EXISTS(SELECT PRODUTO.CDPRODUTO
                                       FROM PRODUTO
                                       WHERE ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO
                                             AND ITEMCOLE.NRORG = PRODUTO.NRORG
                                             AND (PRODUTO.CDARVPROD BETWEEN :P_CDINI
                                             AND :P_CDFIN OR :P_TODOS = 'T'))";

            $params = [
                "P_PORCENT"     => $data["REAJUSTE"],
                "P_CDFILCENT"   => $data["CDFILCENT"],
                "P_NRCOLEPRECO" => $data["NRCOLEPRECO"],
                "P_CDINI"       => $data["CDPRODINI"],
                "P_CDFIN"       => $data["CDPRODFIN"],
                "P_TODOS"       => $data["TODOSPROD"],
                "P_NRORG"       => $data["NRORG"]
            ];

            $this->util->crud($query, $params);
        } catch (Exception $exception) {
            throw new \Exception("Erro ao atualizar data preço: " . $exception->getMessage(), 500);
        }
    }
}
