<?php

namespace SupNext\Services\QuoteSheetGrid;

use SupNext\Util\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class SaveAdditionalDataService {
  private $util;

  public function __construct(Util $util) {
    $this->util = $util;
  }

  public function deletePaymentDeadline(array $data): void {
    try {
      $query = "DELETE FROM PZOPGTITMCOL
                WHERE PZOPGTITMCOL.CDREGIAO = :P_CDREGIAO
                  AND PZOPGTITMCOL.CDFILIAL = :P_CDFILIAL
                  AND PZOPGTITMCOL.NRCOLEPRECO = :P_NRCOLEPRECO
                  AND PZOPGTITMCOL.CDFORNECED = :P_CDFORNECED
                  AND PZOPGTITMCOL.CDPRODUTO = :P_CDPRODUTO
                  AND PZOPGTITMCOL.CDMARCA = LPAD(:P_CDMARCA, 5, '0')
                  AND PZOPGTITMCOL.NRORG = :P_NRORG";

      $params = [
        "P_CDREGIAO"    => $data["CDREGIAO"],
        "P_CDFILIAL"    => $data["CDFILIAL"],
        "P_NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "P_CDFORNECED"  => $data["CDFORNECED"],
        "P_CDPRODUTO"   => $data["CDPRODUTO"],
        "P_CDMARCA"     => $data["CDMARCA"],
        "P_NRORG"       => $data["NRORG"]
      ];

      $this->util->crud($query, $params);
    } catch (Exception $exception) {
      throw new \Exception("Erro ao deletar prazo pagamento: " . $exception->getMessage(), 500);
    }
  }


  public function updateItemCole(array $data): void {
    try {
        $query = "UPDATE ITEMCOLE
                  SET CDFILIAL = :CDFILIAL,
                      NRCOLEPRECO = :NRCOLEPRECO,
                      CDFORNECED = :CDFORNECED,
                      CDREGIAO = :CDREGIAO,
                      CDPRODUTO = :CDPRODUTO,
                      CDMARCA = :CDMARCA,
                      CDTIPCOB = :CDTIPCOB,
                      CDSEGMARCCOL = :CDSEGMARCCOL,
                      DTINIVALITM = :DTINIVALITM,
                      DTFINVALITM = :DTFINVALITM,
                      VRPRECUNIITM = :VRPRECUNIITM,
                      VRPRECECOITM = :VRPRECECOITM,
                      IDTIPFREITM = :IDTIPFREITM,
                      NRPZOENTCOL = :NRPZOENTCOL,
                      CDFORMPGTO = :CDFORMPGTO,
                      IDTPFOPGTCOL = :IDTPFOPGTCOL,
                      NRPARCPGTCOL = :NRPARCPGTCOL,
                      NRDIAPGFXCOL = :NRDIAPGFXCOL,
                      IDUTDIAPGCOL = :IDUTDIAPGCOL,
                      QTACMPITMCOL = :QTACMPITMCOL
                  WHERE CDFILIAL = :OLD_CDFILIAL
                        AND NRCOLEPRECO = :OLD_NRCOLEPRECO
                        AND CDFORNECED = :OLD_CDFORNECED
                        AND CDREGIAO = :OLD_CDREGIAO
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND CDMARCA = :OLD_CDMARCA
                        AND NRORG = :P_NRORG";

      $params = [
        "CDFILIAL"        => $data["CDFILIAL"],
        "NRCOLEPRECO"     => $data["NRCOLEPRECO"],
        "CDFORNECED"      => $data["CDFORNECED"],
        "CDREGIAO"        => $data["CDREGIAO"],
        "CDPRODUTO"       => $data["CDPRODUTO"],
        "CDMARCA"         => $data["CDMARCA"],
        "CDTIPCOB"        => $data["CDTIPCOB"],
        "CDSEGMARCCOL"    => $data["CDSEGMARCCOL"],
        "DTINIVALITM"     => $data["DTINIVALITM"],
        "DTFINVALITM"     => $data["DTFINVALITM"],
        "VRPRECUNIITM"    => $data["VRPRECUNIITM"],
        "VRPRECECOITM"    => $data["VRPRECECOITM"],
        "IDTIPFREITM"     => $data["IDTIPFREITM"],
        "NRPZOENTCOL"     => $data["NRPZOENTCOL"],
        "CDFORMPGTO"      => $data["CDFORMPGTO"],
        "IDTPFOPGTCOL"    => $data["IDTPFOPGTCOL"],
        "NRPARCPGTCOL"    => $data["NRPARCPGTCOL"],
        "NRDIAPGFXCOL"    => $data["NRDIAPGFXCOL"],
        "IDUTDIAPGCOL"    => $data["IDUTDIAPGCOL"],
        "QTACMPITMCOL"    => $data["QTACMPITMCOL"],
        "OLD_CDFILIAL"    => $data["CDFILIAL"],
        "OLD_NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "OLD_CDFORNECED"  => $data["CDFORNECED"],
        "OLD_CDREGIAO"    => $data["CDREGIAO"],
        "OLD_CDPRODUTO"   => $data["CDPRODUTO"],
        "OLD_CDMARCA"     => $data["CDMARCA"],
        "P_NRORG"         => $data["NRORG"]
      ];

      $this->util->crud($query, $params);
    } catch (Exception $exception) {
      throw new \Exception("Erro ao atualizar item cole: " . $exception->getMessage(), 500);
    }
  }

  public function updateFreightSupplier(array $data): void {
    try {
      $query = "UPDATE ITEMCOLE
                SET VRFRETEFORNEPLANCOT = :P_VRFRETEFORN
                WHERE CDFILIAL = :P_CDFILIAL
                      AND NRCOLEPRECO = :P_NRCOLEPRECO
                      AND CDFORNECED = :P_CDFORNECED
                      AND NRORG = :P_NRORG";

      $params = [
        "P_VRFRETEFORN" => (float) $data["VRFRETEFORN"],
        "P_CDFILIAL"    => $data["CDFILIAL"],
        "P_NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "P_CDFORNECED"  => $data["CDFORNECED"],
        "P_NRORG"       => $data["NRORG"]
      ];

      $this->util->crud($query, $params);
    } catch (Exception $exception) {
      throw new \Exception("Erro ao atualizar prazo fornecedor: " . $exception->getMessage(), 500);
    }
  }

  public function generateUpdateQuery($data) {
    $query = "UPDATE ITEMCOLE SET VRPERDESITM = :VRPERDESITM
              WHERE CDFILIAL = :OLD_CDFILIAL
              AND NRCOLEPRECO = :OLD_NRCOLEPRECO";

    if ($data['REPLICADESCONTO'] == 'N') {
      if ($data['IDDESCPROD'] == 1) {
        $query = $query . " AND CDPRODUTO = :P_CDPRODUTO";
      } else {
        $query = $query . " AND INSTR(:P_CDPRODUTO, CDPRODUTO) > 0";
      }

      if ($data['IDDESCMARCA'] == 1) {
        $query = $query . " AND CDMARCA = :P_CDMARCA";
      } else {
        $query = $query . " AND INSTR(:P_CDMARCA, CDMARCA) > 0";
      }

      if ($data['IDDESCREG'] == 1) {
        $query = $query . " AND CDREGIAO = :P_CDREGIAO";
      } else {
        $query = $query . " AND INSTR(:P_CDREGIAO, CDREGIAO) > 0";
      }

      if ($data['IDDESCFORN'] == 1) {
        $query = $query . " AND CDFORNECED = :P_CDFORNECED";
      } else {
        $query = $query . " AND INSTR(:P_CDFORNECED, CDFORNECED) > 0";
      }
    } else {
      if ($data['RDPROD'] == 'M') {
        $query = $query . " AND CDPRODUTO = :P_CDPRODUTO";
      } else if ($data['RDPROD'] == 'T') {
        $query = $query . " AND :P_CDPRODUTO = 'T'";
      } else {
        $query = $query . " AND INSTR(:P_CDPRODUTO, CDPRODUTO) > 0";
      }

      if ($data['RDMARCA'] == 'M') {
        $query = $query . " AND CDMARCA = :P_CDMARCA";
      } else if ($data['RDMARCA'] == 'T') {
        $query = $query . " AND :P_CDMARCA = 'T'";
      } else {
        $query = $query . " AND INSTR(:P_CDMARCA, CDMARCA) > 0";
      }

      if ($data['RDREGIAO'] == 'M') {
        $query = $query . " AND CDREGIAO = :P_CDREGIAO";
      } else if ($data['RDREGIAO'] == 'T') {
        $query = $query . " AND :P_CDREGIAO = 'T'";
      } else {
        $query = $query . " AND INSTR(:P_CDREGIAO, CDREGIAO) > 0";
      }

      if ($data['RDFORNECED'] == 'M') {
        $query = $query . " AND CDFORNECED = :P_CDFORNECED";
      } else if ($data['RDFORNECED'] == 'T') {
        $query = $query . " AND :P_CDFORNECED = 'T'";
      } else {
        $query = $query . " AND INSTR(:P_CDFORNECED, CDFORNECED) > 0";
      }
    }

    return $query;
  }

  // public function generateUpdateQuery(array $data) {
  //   $query = "UPDATE ITEMCOLE SET VRPERDESITM = :VRPERDESITM
  //             WHERE CDFILIAL = :OLD_CDFILIAL
  //             AND NRCOLEPRECO = :OLD_NRCOLEPRECO";

  //   $conditions = [
  //     'CDPRODUTO' => $this->generateCondition($data, 'P_CDPRODUTO', 'IDDESCPROD', 'RDPROD'),
  //     'CDMARCA' => $this->generateCondition($data, 'P_CDMARCA', 'IDDESCMARCA', 'RDMARCA'),
  //     'CDREGIAO' => $this->generateCondition($data, 'P_CDREGIAO', 'IDDESCREG', 'RDREGIAO'),
  //     'CDFORNECED' => $this->generateCondition($data, 'P_CDFORNECED', 'IDDESCFORN', 'RDFORNECED')
  //   ];

  //   foreach ($conditions as $column => $condition) {
  //     if ($condition) {
  //       $query .= " AND {$column} {$condition}";
  //     }
  //   }

  //   return $query;
  // }

  // private function generateCondition(array $data, string $param, string $idDescKey, string $rdKey) {
  //   if ($data['REPLICADESCONTO'] == 'N') {
  //     return $this->buildCondition($param, $data[$idDescKey] == 1);
  //   }

  //   return $this->buildCondition($param, $data[$rdKey] == 'M', $data[$rdKey] == 'T');
  // }

  // private function buildCondition(string $param, $isExactMatch, $isAllMatch = false) {
  //   if ($isAllMatch) {
  //     return "= 'T'";
  //   }

  //   return $isExactMatch ? "= :{$param}" : "AND INSTR(:{$param}, {$param}) > 0";
  // }

  public function updateItemColeCustom($query, $data) {
    try {
      if ($data['REPLICADESCONTO'] == 'N') {
        $produto = $data['CDPRODUTO'];
        $marca = $data['CDMARCA'];
        $regiao = $data['CDREGIAO'];
        $fornecedor = $data['CDFORNECED'];
      } else {
        // Se M só o item selecionado, se T todos os itens e se L considerar a lista.

        if ($data["RDPROD"] == "M") {
          $produto = $data["CDPRODUTO"];
        } else if ($data["RDPROD"] == "T") {
          $produto = "T";
        } else if ($data["RDPROD"] == "L") {
          $produto = $data["CDPRODUTOPES"];
        }

        if ($data['RDMARCA'] == 'M') {
          $marca = $data['CDMARCA'];
        } else if ($data['RDMARCA'] == 'T') {
          $marca = 'T';
        } else if ($data['RDMARCA'] == 'L') {
          $marca = $data['CDMARCAPES'];
        }

        if ($data['RDREGIAO'] == 'M') {
          $regiao = $data['CDREGIAO'];
        } else if ($data['RDREGIAO'] == 'T') {
          $regiao = 'T';
        } else if ($data['RDREGIAO'] == 'L') {
          $regiao = $data['CDREGIAOPES'];
        }

        if ($data['RDFORNECED'] == 'M') {
          $fornecedor = $data['CDFORNECED'];
        } else if ($data['RDFORNECED'] == 'T') {
          $fornecedor = 'T';
        } else if ($data['RDFORNECED'] == 'L') {
          $fornecedor = $data['CDFORNECEDPES'];
        }
      }

      $data1 = [
        "VRPERDESITM"     => $data["VRPERDESITM"],
        "OLD_CDFILIAL"    => $data["CDFILIAL"],
        "OLD_NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "P_CDPRODUTO"     => $produto,
        "P_CDMARCA"       => $marca,
        "P_CDREGIAO"      => $regiao,
        "P_CDFORNECED"    => $fornecedor
      ];

      $this->util->crud($query, $data1);

      $query2 = "UPDATE PARASUPRI
                SET RDPROD = :RDPROD,
                    RDMARCA = :RDMARCA,
                    RDREGIAO = :RDREGIAO,
                    RDFORNECED = :RDFORNECED
                WHERE CDFILIAL = :CDFILIAL
                      AND NRORG = :NRORG";

      $data2 = [
        "RDPROD"     => $data["RDPROD"],
        "RDMARCA"    => $data["RDMARCA"],
        "RDREGIAO"   => $data["RDREGIAO"],
        "RDFORNECED" => $data["RDFORNECED"],
        "CDFILIAL"   => $data["CDFILIAL"],
        "NRORG"      => $data["NRORG"]
      ];

      var_dump($data2);return;

      $this->util->crud($query2, $data2);
    } catch (Exception $exception) {
      throw new \Exception("Erro ao atualizar itemcole: " . $exception->getMessage(), 500);
    }
  }

  // public function updateItemColeCustom($query, array $data) {
  //   try {
  //     $product = $this->determineValue($data, "CDPRODUTO", "RDPROD", "CDPRODUTOPES");
  //     $brand = $this->determineValue($data, "CDMARCA", "RDMARCA", "CDMARCAPES");
  //     $region = $this->determineValue($data, "CDREGIAO", "RDREGIAO", "CDREGIAOPES");
  //     $supplier = $this->determineValue($data, "CDFORNECED", "RDFORNECED", "CDFORNECEDPES");

  //     $data1 = [
  //       "VRPERDESITM"     => $data["VRPERDESITM"],
  //       "OLD_CDFILIAL"    => $data["CDFILIAL"],
  //       "OLD_NRCOLEPRECO" => $data["NRCOLEPRECO"],
  //       "P_CDPRODUTO"     => $product,
  //       "P_CDMARCA"       => $brand,
  //       "P_CDREGIAO"      => $region,
  //       "P_CDFORNECED"    => $supplier
  //     ];

  //     $this->util->crud($query, $data1);

  //     $query2 = "UPDATE PARASUPRI
  //               SET RDPROD = :RDPROD,
  //                   RDMARCA = :RDMARCA,
  //                   RDREGIAO = :RDREGIAO,
  //                   RDFORNECED = :RDFORNECED
  //               WHERE CDFILIAL = :CDFILIAL
  //                     AND NRORG = :NRORG";

  //     $data2 = [
  //       "RDPROD"     => $data["RDPROD"],
  //       "RDMARCA"    => $data["RDMARCA"],
  //       "RDREGIAO"   => $data["RDREGIAO"],
  //       "RDFORNECED" => $data["RDFORNECED"],
  //       "CDFILIAL"   => $data["CDFILIAL"],
  //       "NRORG"      => $data["NRORG"]
  //     ];

  //     $this->util->crud($query2, $data2);
  //   } catch (Exception $exception) {
  //     throw new \Exception("Erro ao atualizar itemcole: " . $exception->getMessage(), 500);
  //   }
  // }

  // private function determineValue(array $data, string $defaultKey, string $replicaKey, string $searchKey) {
  //   if ($data['REPLICADESCONTO'] == 'N') {
  //     return $data[$defaultKey];
  //   }

  //   switch ($data[$replicaKey]) {
  //     case 'M':
  //       return $data[$defaultKey];
  //     case 'T':
  //       return 'T';
  //     case 'L':
  //       return $data[$searchKey];
  //     default:
  //       return null;
  //   }
  // }
}
