import {
  Form, ModalService, Select, LoadingService,
} from '@zeedhi/common';
import { Metadata, Http } from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';

export class ValidateBuyerController {
  private utilities!: Utilities;

  private static quoteOperator;

  constructor() {
    this.utilities = new Utilities();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/validateBuyerModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('validateBuyerModal')?.destroy();
  }

  public static openModal(): void {
    ModalService.getModal('validateBuyerModal').show();
  }

  public closeModal(): void {
    ModalService.getModal('validateBuyerModal').hide();

    const quoteOperator = Metadata.getInstance<Select>('quoteOperator');
    quoteOperator.value = null;
  }

  public static setUser(): void {
    setTimeout(() => {
      const form = Metadata.getInstance<Form>('validateBuyerForm');
      const quoteOperator = Metadata.getInstance<Select>('quoteOperator');
      form.value.user = `${quoteOperator.value.CDOPERADOR} | ${quoteOperator.value.NMOPERADOR}`;
    }, 100);
  }

  public async validateBuyer(): Promise<void> {
    const form = Metadata.getInstance<Form>('validateBuyerForm');
    const quoteOperator = Metadata.getInstance<Select>('quoteOperator');

    if (!form.value.password) {
      this.utilities.showDialog('Senha deve ser preenchida', 'error');

      return;
    }

    const params = {
      CDOPERADOR: quoteOperator.value.CDOPERADOR,
      PASSWORD: form.value.password,
    };

    LoadingService.show();

    try {
      const { data } = await Http.post('/validateBuyer', params);

      if (data.error) {
        LoadingService.hide();
        this.utilities.showDialog(data.error, 'error');

        return;
      }

      LoadingService.hide();
      form.clearForm();
      ModalService.getModal('validateBuyerModal').hide();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }
}
